#!/usr/bin/perl -s
#
# xcalendar.pl and xcalendar(-j).cgi - support Compact HTML, MML, HDML
#	by Taiji Yamada at 5/12/2000, 8/3/2000, 11/28/2000
# xcalendar.pl and xcalendar(-j).cgi - support PostScript printing
#	by Taiji Yamada at 12/10/1999, 12/10/2001
# xcalendar(-j).cgi - compatible with xcalendar except for data editting
#	by Taiji Yamada at 9/24/1999
# xc2html(-j).cgi - simplified and refined for languages, dating and so on
#	by Taiji Yamada at 9/22/1999
# xc2html.cgi - improved at holiday files according to our private calendar.cgi
#	by Jun Takahashi at 9/21/1999
# xc2html.pl - display xcalendar data in HTML
#	by Shunsuke Horai
#
$version = '0.9997';
$mailto = 'taiji@aihara.co.jp';
$copyright = 'Copyright (C) 1999-2001 Taiji Yamada, Jun Takahashi, Shunsuke Horai';

use POSIX;
use Time::Local;

#
# in the directory of this script, please `ln -s ~/Calendar .` for CGI.
#
$xc_file_f = 'Calendar/xc%d%h%Y';

#
# xcalendar holiday file format
#
# month/day/year	holiday label
# 12/25/1999		X'mas
#	;
$xcalendar_hol = "/usr/X11R6/lib/X11/xcalendar/xcalendar.hol";
$xcalendar_hol2 = "/usr/X11R6/lib/X11/xcalendar/xcalendar.hol2" if (!$nohol2);

#
# simple-style holiday file format
#
# year	month	holiday list		2nd holiday list
# 1999,	1,	1,3,10,15,17,24,31,	2,3,4,13,23,30
#	;
$holidays_txt = "/opt/local/share/holidays.txt" if (!$noholx);

#
# subroutines to check special browsers
#
sub is_chtml { # for Compact HTML; i-mode(NTT-DoCoMo)
  if ($ENV{"HTTP_USER_AGENT"} =~ /^DoCoMo/i) {
    if ($ENV{"HTTP_USER_AGENT"} =~ /^DoCoMo\/[^\/]+\/(.*)/i) {
      $disable_table = !0, $n_notab = 2 if ($1 =~ /^.501i$/);
    }
    !0;
  }
  else {
    0;
  }
}
sub is_mml { # for MML(Mobile Markup Language); J-SkyWeb(J-Phone)
  if (($ENV{"HTTP_USER_AGENT"} eq '-' &&
       $ENV{"REMOTE_HOST"} =~ /^.*\.skyweb\..*\.ne\.jp$/i)) {
    $disable_colspan = !0;
    !0;
  }
  else {
    0;
  }
}
sub is_hdml { # for HDML(Handheld Device Markup Language); EZ-Web(IDO, DDI)
  if ($ENV{"HTTP_USER_AGENT"} =~ /^UP.Browser/i) {
    $disable_table = !0, $n_notab = 3;
    !0;
  }
  else {
    0;
  }
}

#
# settings for languages (Japanese or English) in HTML or PostScript
#
$me = `basename $0`; chomp($me);
$me = $ENV{"SCRIPT_NAME"}?$ENV{"SCRIPT_NAME"}:$me;
if ($me =~ /.cgi$/ || $ENV{"HTTP_HOST"}) {
  $content_type = is_hdml()?'text/x-hdml':'text/html';
  if ($me =~ /-j.cgi$/) {
    $bin_nkf = '/opt/local/bin/nkf'; # the pathname of Network Kanji Filter
    #
    # set locale for Japanese coding system
    #
    setlocale(LC_ALL, ($locale="ja"));
    #$charset = "iso-2022-jp",	$out_filter = "|$bin_nkf -j";
    $charset = "euc-jp",	$out_filter = "";
    #$charset = "shift_jis",	$out_filter = "|$bin_nkf -s";
    if (is_chtml() || is_mml() || is_hdml()) { # depending upon a browser
      $charset = "shift_jis",	$out_filter = "|$bin_nkf -s";
    }
    @week_name = ("", "", "", "", "", "", "");
    $title_f = "%Yǯ %B";
    $date_f = "%x %a";
  }
  else {
    #
    # set locale to C
    #
    setlocale(LC_ALL, ($locale="C"));
    $charset = "iso-8859-1", $out_filter = "";
    @week_name = ("S", "M", "Tu", "W", "Th", "F", "S"); # `cal` style
    @week_name = ("Su", "Mo", "Tu", "We", "Th", "Fr", "Sa") if ($disable_table);
    $title_f = "%B %Y";
    $date_f = "%A %B %d %Y";
  }
  #
  # buttons for actions and misc.
  #
  #$prev_month_button = "<img SRC=\"larrow.xbm\" ALT=\"<\" BORDER=0>";
  #$next_month_button = "<img SRC=\"rarrow.xbm\" ALT=\">\" BORDER=0>";
  $prev_month_button = "&lt;";
  $next_month_button = "&gt;";
  $bg_image = "xcalendar-bg.gif"; $bg_image = '' if (! -f $bg_image);
  #$bg_image = "";
  $tbl_border = 1, $tbl_cellspacing = 1, $tbl_cellpadding = 2;
  if (is_chtml() || is_mml() || is_hdml()) { # depending upon a browser
    $bg_image = "";
    $tbl_border = 0, $tbl_cellspacing = 0, $tbl_cellpadding = 0;
  }
  @html_colors = ("black", "red", "blue", "#CCFFCC", "#FFFFCC");
  $month_navigate_flag = !0;
  $caldata_flag = !0;
  $holdata_flag = !0;
  $months_cols = 4;
  $months_cols = 1 if (is_chtml() || is_mml() || is_hdml());
}
else {
  $content_type = 'application/postscript';
  if ($ENV{"LANG"} =~ /^ja/) {
    #
    # set charset and filter for Japanese coding system and PostScript
    #
    #$charset = "iso-2022-jp",	$out_filter = "|$bin_nkf -j";
    $charset = "euc-jp",	$out_filter = "";
    #$charset = "shift_jis",	$out_filter = "|$bin_nkf -s";
    @week_name = ("", "", "", "", "", "", "");
    $title_f = "%Yǯ %B";
    $date_f = "%x %a";
    #
    # fonts for Japanese, requires composite fonts
    #
    @ps_fs_title = ('Bookman-Light',	'Ryumin-Light-EUC-H');
    @ps_fs_week  = ('AvantGarde-Book',	'GothicBBB-Medium-EUC-H');
    @ps_fs_day   = ('Palatino-Roman',	'Ryumin-Light-EUC-H');
    @ps_fs_body  = ('Times-Roman',	'Ryumin-Light-EUC-H');
    @ps_fs_foot  = ('Courier',		'GothicBBB-Medium-EUC-H');
  }
  else {
    $charset = "default",       $out_filter = "";
    @week_name = ("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat");
    $title_f = "%B %Y";
    $date_f = "%A %B %d %Y";
    #
    # fonts for C locale, but encoding depends upon Adobe Standard Encoding 
    #
    @ps_fs_title = ('Bookman-Light');
    @ps_fs_week  = ('AvantGarde-Book');
    @ps_fs_day   = ('Palatino-Roman');
    @ps_fs_body  = ('Times-Roman');
    @ps_fs_foot  = ('Courier');
  }
  $ps_title = "$me @ARGV";
  $ps_color_mode = !0;	# default color
  $ps_color_mode = 0 if ($mono); # -mono
  $ps_color_mode = !0 if ($color); # -color
  @ps_colors = ("0", "1 0 0", "0.5 0.5 1");
  #
  # PostScript printer characteristics
  #
  %ps_device_origin =
      (# printer product name => [ left(72inch), bottom(72inch) ],
       'Canon_A404_PS' => [ +6, -6 ],
       'Phaser_750'    => [ +0, -1 ],
       'GhostScript'   => [ +0, +0 ],
       # please insert the entry of the gap(72inch) between the PostScript
       # origin and the origin of a paper for your PostScript printer, here.
       );
  %ps_device_printable_margins =
      (# printer product name => [ left(72inch), top(72inch),
       #                          right(72inch), bottom(72inch) ],
       'Canon_A404_PS' => [ 13, 10, 10, 10 ],
       'Phaser_750'    => [ 14, 13, 18,  7 ],
       'GhostScript'   => [ 50, 50, 40, 40 ],
       # please insert the entry of the margins which your PostScript printer
       # can paint on a paper, here.
       );
  $ps_printer_name = 'Canon_A404_PS';
  #$ps_printer_name = 'please write the name of your PostScript printer, here
  $ps_printer_name = $PS if ($PS); # -PS='GhostScript'
  #
  # PostScript media types, orientation and margin
  #
  %media =
      (# type => [ width(72inch), height(72inch) ], # 25.4(mm/inch)
       'Ledger'    => [ 1224,  792 ], # 17  (inch)x11  (inch)
       'Tabloid'   => [  792, 1224 ], # 11  (inch)x17  (inch)
       'Letter'    => [  612,  792 ], #  8.5(inch)x11  (inch)
       'Statement' => [  396,  612 ], #  5.5(inch)x 8.5(inch)
       'Legal'     => [  612, 1008 ], #  8.5(inch)x14  (inch)
       'Executive' => [  540,  720 ], #  7.5(inch)x10  (inch)
       'A0'        => [ 2384, 3370 ], #  841(mm)x1189(mm)
       'A1'        => [ 1684, 2384 ], #  594(mm)x 841(mm)
       'A2'        => [ 1190, 1684 ], #  420(mm)x 594(mm)
       'A3'        => [  842, 1190 ], #  297(mm)x 420(mm)
       'A4'        => [  595,  842 ], #  210(mm)x 297(mm)
       'A5'        => [  420,  595 ], #  148(mm)x 210(mm)
       'A6'        => [  298,  420 ], #  105(mm)x 148(mm)
       'B0'        => [ 2920, 4127 ], # 1030(mm)x1456(mm)
       'B1'        => [ 2064, 2920 ], #  728(mm)x1030(mm)
       'B2'        => [ 1460, 2064 ], #  515(mm)x 728(mm)
       'B3'        => [ 1032, 1460 ], #  364(mm)x 515(mm)
       'B4'        => [  729, 1032 ], #  257(mm)x 364(mm)
       'B5'        => [  516,  729 ], #  182(mm)x 257(mm)
       'B6'        => [  363,  516 ], #  128(mm)x 182(mm)
       'Folio'     => [  612,  936 ], # 8.5(inch)x13(inch)
       'Quarto'    => [  610,  780 ], #
       '10x14'     => [  720, 1008 ], # 10(inch)x14(inch)
       'Bible2'    => [  482,  538 ], # 170(mm)x190(mm)
       'Bible'     => [  269,  482 ], #  95(mm)x170(mm)
       'Mini2'     => [  357,  454 ], # 126(mm)x160(mm)
       'Mini'      => [  227,  357 ], #  80(mm)x126(mm)
       'Pocket2'   => [  357,  430 ], # 126(mm)x152(mm)
       'Pocket'    => [  215,  357 ], #  76(mm)x126(mm)
       );
  @orientations = ('Portrait', 'Landscape');
  %margins =
      (# type => [ left(72inch), top(72inch), right(72inch), bottom(72inch) ]
       'Ledger'    => [ 40, 40, 40, 40 ],
       'Tabloid'   => [ 40, 40, 40, 40 ],
       'Letter'    => [ 40, 40, 40, 40 ],
       'Statement' => [ 40, 40, 40, 40 ],
       'Legal'     => [ 40, 40, 40, 40 ],
       'Executive' => [ 40, 40, 40, 40 ],
       'A0'        => [ 40, 40, 40, 40 ],
       'A1'        => [ 40, 40, 40, 40 ],
       'A2'        => [ 40, 40, 40, 40 ],
       'A3'        => [ 40, 40, 40, 40 ],
       'A4'        => [ 40, 40, 40, 40 ], # about 14(mm)
       'A5'        => [ 40, 40, 40, 40 ],
       'A6'        => [ 40, 40, 40, 40 ],
       'B0'        => [ 40, 40, 40, 40 ],
       'B1'        => [ 40, 40, 40, 40 ],
       'B2'        => [ 40, 40, 40, 40 ],
       'B3'        => [ 40, 40, 40, 40 ],
       'B4'        => [ 40, 40, 40, 40 ],
       'B5'        => [ 40, 40, 40, 40 ],
       'B6'        => [ 40, 40, 40, 40 ],
       'Folio'     => [ 40, 40, 40, 40 ],
       'Quarto'    => [ 40, 40, 40, 40 ],
       '10x14'     => [ 40, 40, 40, 40 ],
       'Bible2'    => [ 21, 21, 21, 21 ], # about 7.5(mm)
       'Bible'     => [ 29, 21, 21, 21 ], # about 10(mm) and 7.5(mm)
       'Mini2'     => [ 19, 19, 19, 19 ], # about 6.5(mm)
       'Mini'      => [ 29, 19, 19, 19 ], # about 10(mm) and 6.5(mm)
       'Pocket2'   => [ 19, 19, 19, 19 ], # about 6.5(mm)
       'Pocket'    => [ 29, 19, 19, 19 ], # about 10(mm) and 6.5(mm)
       );
  $medium = 'A4';
  #$medium = 'Mini';
  $orientation = 0;
  $orientation = 1 if ($landscape); # -landscape
  if ($paper) { # -paper='media type'
    substr($paper,0,1) =~ tr/a-z/A-Z/;
    if (@{$media{$paper}} > 0) {
      $medium = $paper;
    }
    elsif ($size =~ /^\d+[x,]\d+$/) { # -size='227x363'
      @{$media{($medium = $paper)}} = split(/[x,]/, $size);
      @{$margins{$medium}} = @{$ps_device_printable_margins{$ps_printer_name}};
    }
  }
  @{$margins{$medium}} = split(/,/, $margin)
      if ($margin =~ /^\d+,\d+,\d+,\d+$/); # -margin='50,40,30,30'
  %half_media =
      (# type => type of half size
       'Letter'=>'Statement', 'Tabloid'=>'Letter',
       'A0'=>'A1', 'A1'=>'A2', 'A2'=>'A3', 'A3'=>'A4', 'A4'=>'A5', 'A5'=>'A6',
       'B0'=>'B1', 'B1'=>'B2', 'B2'=>'B3', 'B3'=>'B4', 'B4'=>'B5', 'B5'=>'B6',
       'Bible2'=>'Bible', 'Mini2'=>'Mini', 'Pocket2'=>'Pocket',
       );
  %margin_folds =
      (# type => [ various margins(72inch,...) ]
       #'letter'     => [], # u-bottom, u-top, l-bottom, l-top
       #'z-fold'     => [], # u-bottom, u-top, l-bottom, l-top
       #'doublegate' => [], # l-right, l-left, m-right, m-left, r-right, r-left
       #'leftgate'   => [], # l-right, l-left, r-right, r-left
       #'rightgate'  => [], # l-right, l-left, r-right, r-left
       'saddle'     => [ 29, 29 ], # m-right, m-left
       );
  if ($fold eq 'saddle') { # -fold=saddle
    $orientation = 1;
    @{$margin_folds{$fold}} =
        ($margins{$half_media{$medium}}[0], $margins{$half_media{$medium}}[0])
            if ($half_media{$medium});
  }
  #
  # BoundingBox of (device,) all pages and a page
  #
  @dbb = (0, $media{$medium}[1], $media{$medium}[0], 0);
  @bb = (0 + $margins{$medium}[0],
         $media{$medium}[1] - $margins{$medium}[1],
         $media{$medium}[0] - $margins{$medium}[2],
         0 + $margins{$medium}[3]);
  @pbb = @bb;
  if ($orientation == 1) {
    $ps_recoord_op =
        "$ps_device_origin{$ps_printer_name}[0] $media{$medium}[0] add ".
            "$ps_device_origin{$ps_printer_name}[1] translate 90 rotate";
    @DBB = reverse(@dbb);
    @BB = reverse(@bb);
    @PBB = reverse(@pbb);
  }
  else {
    $ps_recoord_op =
        "$ps_device_origin{$ps_printer_name}[0] $ps_device_origin{$ps_printer_name}[1] translate";
    @DBB = @dbb;
    @BB = @bb;
    @PBB = @pbb;
  }
  #
  # title position and BoundingBox of the calendar
  #
  $ps_hskp_ratio_W = 0.02;
  $ps_vskp_ratio_H = 0.01;
  $ps_title_ratio_H = 0.06;
  $ps_week_ratio_H = 0.04;
  @ps_day_ratio_box = (0.3, 0.15);
  @ps_title_pos = ($PBB[0] + $ps_hskp_ratio_W*($PBB[2]-$PBB[0]+0),
                   $PBB[1] - ($ps_title_ratio_H - $ps_vskp_ratio_H/2)
                   *($PBB[1]-$PBB[3]+0));
  @ps_foot_pos = ($PBB[2], $PBB[3]-$margins{$medium}[3]/3);
  @cbb_ratio_bb = (1, 0.9);
  if (!$fold) {
    @day_box_size = (($BB[2]-$BB[0]+0)*$cbb_ratio_bb[0]/7,
                     ($BB[1]-$BB[3]+0)*$cbb_ratio_bb[1]/6);
    @cbb = ($PBB[0], $PBB[3] + 6*$day_box_size[1], $PBB[2], $PBB[3]);
  }
  elsif ($fold eq 'saddle') {
    @day_box_size = ((($BB[2]-$BB[0]+0)/2 - $margin_folds{$fold}[0])
                     *$cbb_ratio_bb[0]/4,
                     ($BB[1]-$BB[3]+0)*$cbb_ratio_bb[1]/6);
    @cbb = ($PBB[0], $PBB[3] + 6*$day_box_size[1], $PBB[2], $PBB[3]);
    @lbb = ($cbb[0], $cbb[1],
            ($DBB[2]-$DBB[0]+0)/2 - $margin_folds{$fold}[0],
            $cbb[3]);
    @rbb = (($DBB[2]-$DBB[0]+0)/2 + $margin_folds{$fold}[1],
            $cbb[1], $cbb[2], $cbb[3]);
  }
  #
  # preferable font sizes and font names
  #
  $ps_fn_title_size = (($ps_title_ratio_H-$ps_vskp_ratio_H)*($BB[1]-$BB[3]+0));
  $ps_fn_week_size = (($ps_week_ratio_H-$ps_vskp_ratio_H)*($BB[1]-$BB[3]+0));
  $ps_fn_day_size = ($day_box_size[0]/($day_fn_scale?$day_fn_scale:10/3));
  $ps_fn_body_size = ($day_box_size[0]/($body_fn_scale?$body_fn_scale:10));
  $ps_fn_foot_size = ($margins{$medium}[3]/3);
  if ($charset =~ /^euc-/) {
    $ps_fn_title = "$ps_fs_title[0],$ps_fs_title[1]";
    $ps_fn_week = "$ps_fs_week[0],$ps_fs_week[1]";
    $ps_fn_day = "$ps_fs_day[0],$ps_fs_day[1]";
    $ps_fn_body = "$ps_fs_body[0],$ps_fs_body[1]";
    $ps_fn_foot = "$ps_fs_foot[0],$ps_fs_foot[1]";
  }
  else {
    $ps_fn_title = "$ps_fs_title[0]";
    $ps_fn_week = "$ps_fs_week[0]";
    $ps_fn_day = "$ps_fs_day[0]";
    $ps_fn_body = "$ps_fs_body[0]";
    $ps_fn_foot = "$ps_fs_foot[0]";
  }
  #
  # initial states and misc.
  #
  $ps_page = 0;
  $ps_linewidth = '0.3';
  $caldata_flag = 0;
  $holdata_flag = !0;
}
#
# option to print holiday and calendar data of xcalendar
#
$holdata_flag = !0 if ($holdata);       # -holdata
$holdata_flag = 0 if ($noholdata);      # -noholdata

$caldata_flag = !0 if ($caldata);       # -caldata
$caldata_flag = 0 if ($nocaldata);      # -nocaldata

#
# this day, this year and this month
#
$date = strftime("%d %m %Y", localtime(time()));
($this_day, $this_month, $this_year) = $date =~ /(\d+) (\d+) (\d+)/;
$this_day+=0, $this_month+=0, $this_year+=0;

#
# arguments
#
# $me 28 9 1999, to display the calendar and xcalendar data of the day.
# $me 9 1999,    to display the calendar of the month.
# $me 1999,      to display the calendar of the year.
# $me,           to display the calendar and xcalendar data of this day.
#
if (@ARGV == 3 && "@ARGV" =~ /(\d+) (\d+) (\d+)/) {
  ($day, $month, $year) = ($1, $2, $3);
}
elsif (@ARGV == 2 && "@ARGV" =~ /(\d+) (\d+)/) {
  ($month, $year) = ($1, $2);
}
elsif (@ARGV == 1 && "@ARGV" =~ /(\d+)/) {
  ($year) = ($1);
  $months_flag = !0;
}
else { #elsif (@ARGV == 0) {
  ($day, $month, $year) = $date =~ /(\d+) (\d+) (\d+)/;
}
if ($h || $help) {
  usage(STDOUT);
  exit 0;
}

#
# print usage
#
sub usage {
  my($FP) = @_;
  return if ($ENV{"HTTP_HOST"});
  print $FP <<EOF;
xcalendar(-j).cgi, xcalendar.pl - XCalendar Compatible CGI/PostScript Calendar
Usage:
	CGI)	http://path-to/xcalendar.cgi[?[[<day>]+<month>]+<year>]
	HTML)	xcalendar.cgi [[[<day>] <month>] <year>] > output.html
	HDML)	env HTTP_USER_AGENT=UP.Browser \\
		xcalendar.cgi [[[<day>] <month>] <year>] > output.hdml
	PS)	xcalendar.pl [options] [[<month>] <year>] | lpr

Options:
	PS)
	-mono		print in monochrome PostScript mode.
	-color		print in color PostScript mode, default.
	-landscape	print in landscape mode.
	-paper=<medium> select or use the name of medium as follows:
		letter,tabloid,ledger,legal,statement,executive,
		a3,a4,a5,b4,b5,
		folio,quarto,10x14,
		bible,mini,pocket,
			and user defined medium by following options.
	-size=<width>x<height>	set width and height of the medium.
	-margin=<left>,<top>,<right>,<bottom>	set margins of the medium.
	-fold=<type>	set fold type as follows:
		(letter,) (z-fold,)
		(doublegate,)
		(leftgate,) (rightgate,)
		saddle,
			and saddle fold automatically set to landscape.
	-holdata|-noholdata	print a calendar with holiday data, or not.
	-caldata|-nocaldata	print a calendar with calendar data, or not.
	-cutline	print with the frame line along which to cut off.
	-foldline	print with the dotted line along which to fold.

Version:	$version
Copyright:	$copyright
Errors-To:	$mailto
EOF
}

#
# convenience routine to process dating, as a subset of strftime(3C)
#
sub strfdate {
  my($format, $mday, $mon, $myear) = @_;
  strftime($format, localtime(timelocal(0, 0, 0, $mday, $mon, $myear)));
}

#
# routine to check a leap year
#
sub is_leap_year {
  (strfdate("%j", 31, 12-1, @_) == 366);
}

#
# read xcalendar.hol and return holidays in the month
#
sub xc_holidays {
  my($xcalendar_hol, $month, $year) = @_;
  my($data, @data, %hol, $date);
  $month+=0;
  if (-f $xcalendar_hol) {
    $data = `/bin/grep "^0*$month/[0-9][0-9]*/$year\t" $xcalendar_hol`;
    $data .= "\n". `/bin/grep "^0*$month/[0-9][0-9]*\t" $xcalendar_hol`;
    @data = split(/\n+/, $data);
    foreach $data (@data) {
      ($date, $data) = $data =~ m|\d+/(\d+).*\t+(.*)|;
      $hol{$date} = $data;
    }
  }
  %hol;
}

#
# read holidays.txt and return holidays in the month
#
sub holidays {
  my($holidays_txt, $month, $year) = @_;
  my($data, @data);
  $month+=0;
  $data = `/bin/grep "$year,\t\t*$month," $holidays_txt` if (-f $holidays_txt);
  chomp $data;
  @data = split(/\t+/, $data);
  ( [split(',', $data[2])], [split(',', $data[3])] );
}

my(@month_days) = ([ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ],
                   [ 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]);

#
# start main
#
$OUT = STDOUT;
if ($out_filter) {
  open(OUT, "$out_filter");
  $OUT = "OUT";
}
if (!$months_flag) {
  $title = strfdate($title_f, 1, $month-1, $year);
  if ($content_type eq 'text/html') {
    #
    # monthly calendar in HTML
    #
    html_print_header($title);
    html_print_the_month($month, $year, $title) if (!$disable_table);
    html_notab_print_the_month($month, $year, $title) if ($disable_table);
    html_print_footer();
  }
  elsif ($content_type eq 'text/x-hdml') {
    #
    # monthly calendar in HDML
    #
    hdml_print_header($title);
    hdml_print_the_month($month, $year, $title);
    hdml_print_footer();
  }
  else {
    #
    # monthly calendar in PostScript
    #
    ps_print_header($title);
    ps_print_the_month($month, $year, $title);
    ps_print_footer();
  }
}
else {
  $subtitle_f = $title_f;
  $title_f =~ s/ *%[bB] *//g;
  $title = strfdate($title_f, 1, 0, $year);
  if ($content_type eq 'text/html') {
    #
    # yearly calendar in HTML
    #
    $month_navigate_flag = 0;
    html_print_header($title);
    $subtitle_f =~ s/ *%[yY]\S* *//g;
    html_print_header_months($title);
    for ($month=1; $month<=12; $month++) {
      $subtitle = strfdate($subtitle_f, 1, $month-1, $year);
      print $OUT "<td>";
      html_print_the_month($month, $year, $subtitle) if (!$disable_table);
      html_notab_print_the_month($month, $year, $subtitle) if ($disable_table);
      print $OUT "</td>";
      html_print_waister_months() if ($month%$months_cols == 0);
    }
    html_print_footer_months();
    html_print_footer();
  }
  elsif ($content_type eq 'text/x-hdml') {
    #
    # yearly calendar in HDML
    #
    #$month_navigate_flag = 0;
    hdml_print_header($title);
    $subtitle_f =~ s/ *%[yY]\S* *//g;
    hdml_print_header_months($title);
    if (!$divide_deck) {
      for ($month=1; $month<=12; $month++) {
        $subtitle = strfdate($subtitle_f, 1, $month-1, $year);
        hdml_print_the_month($month, $year, $subtitle);
      }
    }
    hdml_print_footer_months();
    hdml_print_footer();
  }
  else {
    #
    # yearly calendar in PostScript
    #
    ps_print_header($title);
    for ($month=1; $month<=12; $month++) {
      $subtitle = strfdate($subtitle_f, 1, $month-1, $year);
      ps_print_the_month($month, $year, $subtitle);
    }
    ps_print_footer();
  }
}
close(OUT) if ($out_filter);

#
# html_print_header(title of page)
#
sub html_print_header {
  my($title) = @_;
  print $OUT "Content-type: $content_type\n\n" if ($ENV{"HTTP_HOST"});
  if (is_chtml()) {
    print $OUT "<!doctype html public \"-//w3c//dtd compact html 1.0 draft//en\">\n";
  }
  elsif (is_mml()) {
  }
  else {
    print $OUT "<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n";
  }
  print $OUT <<EOF;
<html>
<head>
<meta http-equiv="Content-Type" content="$content_type; charset=$charset">
<meta name="GENERATOR" content="$me $version, $mailto">
<meta name="Author" content="$copyright">
<title>XCalendar $title</title>
</head>
EOF
  printf $OUT "<body text=\"$html_colors[0]\" bgcolor=\"white\" link=\"blue\" vlink=\"blue\" alink=\"blue\"%s>\n", $bg_image?" background=\"$bg_image\"":"";
}

#
# html_print_footer()
#
sub html_print_footer {
  print $OUT "</body>\n</html>\n";
}

#
# html_print_header_months(title of page)
#
sub html_print_header_months {
  my($title) = @_;
  print $OUT <<EOF if (!$disable_colspan);
<center>
<table cellpadding=2>
<tr><td colspan=$months_cols align="Center"><font size="+2">$title</font></td></tr>
<tr valign="Top" align="Center">
EOF
  print $OUT <<EOF if ($disable_colspan);
<center>
<table cellpadding=2>
<caption><font size="+2">$title</font></caption>
<tr valign="Top" align="Center">
EOF
}

#
# html_print_waister_months()
#
sub html_print_waister_months {
  print $OUT <<EOF;
</tr>
<tr valign="Top" align="Center">
EOF
}

#
# html_print_footer_months()
#
sub html_print_footer_months {
  print $OUT <<EOF;
</tr>
</table>
</center>
EOF
}

#
# html_print_the_day(day of month, week number)
#
sub html_print_the_day {
  my($d, $w) = @_;
  my($color, $holiday, $xc_file);
  $color = 0;
  $color = 1 if ($w == 0 || $hol{$d} ne "");
  $color = 2 if ($hol2{$d} ne "");
  for (my($i)=0; $i<2; $i++) {
    foreach $holiday (@{$holiday[$i]}) {
      if ($d == $holiday) {
        $color = ($i==0)?1:2;
        last;
      }
    }
  }
  if (($this_year == $year) && ($this_month == $month) && ($this_day == $d)) {
    print $OUT "<td bgcolor=\"$html_colors[3]\">";
  }
  elsif ($day == $d) {
    print $OUT "<td bgcolor=\"$html_colors[4]\">";
  }
  else {
    print $OUT "<td>";
  }
  if ($caldata_flag) {
    setlocale(LC_ALL, "C");
    $xc_file = strfdate($xc_file_f, $d, $month-1, $year), $xc_file =~ s/xc0/xc/;
    setlocale(LC_ALL, $locale);
  }
  print $OUT "<a href=\"$me?$d+$month+$year\">" if ($caldata_flag && -f $xc_file);
  print $OUT "<font color=\"$html_colors[$color]\">$d</font>";
  print $OUT "</a>" if ($caldata_flag && -f $xc_file);
  print $OUT "</td>";
}

#
# html_print_the_month(month, year, title of table)
#
sub html_print_the_month {
  my($month, $year, $title) = @_;
  my($leap_year) = is_leap_year($year);
  %hol = xc_holidays($xcalendar_hol, $month, $year);
  %hol2 = xc_holidays($xcalendar_hol2, $month, $year);
  @holiday = holidays($holidays_txt, $month, $year);

  print $OUT "<center>\n";
  print $OUT "<table border=$tbl_border cellspacing=$tbl_cellspacing cellpadding=$tbl_cellpadding>\n";
  print $OUT "<tr align=\"Center\"><td colspan=7".((($this_year == $year) && ($this_month == $month))?" bgcolor=\"$html_colors[3]\"":"").">" if (!$disable_colspan);
  print $OUT "<caption>" if ($disable_colspan);
  $prev_month = ($month+10)%12 + 1;
  $prev_year = $year - ($prev_month==12?1:0);
  $next_month = $month%12 + 1;
  $next_year = $year + ($next_month==1?1:0);
  print $OUT "<a href=\"$me?$prev_month+$prev_year\"".
      (is_chtml()?" accesskey=\"*\"":is_mml()?" directkey=\"*\"":"").
          ">$prev_month_button</a>" if ($month_navigate_flag);
  print $OUT "$title";
  print $OUT "<a href=\"$me?$next_month+$next_year\"".
      (is_chtml()?" accesskey=\"#\"":is_mml()?" directkey=\"#\"":"").
           ">$next_month_button</a>" if ($month_navigate_flag);
  print $OUT "</td></tr>\n" if (!$disable_colspan);
  print $OUT "</caption>\n" if ($disable_colspan);

  print $OUT "<tr align=\"Center\">\n";
  for (my($w)=0; $w<@week_name; $w++) {
    print $OUT "<td><tt>";
    printf $OUT ($w==0)?"<font color=\"$html_colors[1]\">%s</font>":"%s", $week_name[$w];
    print $OUT "</tt></td>";
  }
  print $OUT "</tr>\n";
  my($md) = $month_days[$leap_year][$month-1];
  my($wd) = strfdate("%w", 1, $month-1, $year);
  for (my($n, $d)=(0, 0); $n<($wd + $md)/7; $n++) {
    print $OUT "<tr align=\"Center\">\n";
    for (my($w)=0; $w<7; $w++) {
      if ($w < $wd && $d == 0) {
        print $OUT "<td>&nbsp;</td>";
      }
      elsif (++$d <= $md) {
        html_print_the_day($d, $w);
      }
      else {
        print $OUT "<td>&nbsp;</td>";
      }
    }
    print $OUT "</tr>\n";
  }
  print $OUT "</table>\n";
  print $OUT "</center>\n";

  if ($day) {
    if ($caldata_flag) {
      setlocale(LC_ALL, "C");
      $xc_file = strfdate($xc_file_f, $day, $month-1, $year), $xc_file =~ s/xc0/xc/;
      setlocale(LC_ALL, $locale);
    }
    printf $OUT "<br>%s\n", strfdate($date_f, $day, $month-1, $year);
    printf $OUT "%s\n", $hol{$day} if ($holdata_flag && $hol{$day});
    printf $OUT "%s\n", $hol2{$day} if ($holdata_flag && $hol2{$day});
    print $OUT "<br><pre>\n".`/bin/cat $xc_file`."\n</pre>\n" if ($caldata_flag && -f $xc_file);
  }
}

#
# html_notab_print_the_day(day of month, week number)
#
sub html_notab_print_the_day {
  my($d, $w) = @_;
  my($color, $holiday, $xc_file);
  $color = 0;
  $color = 1 if ($w == 0 || $hol{$d} ne "");
  $color = 2 if ($hol2{$d} ne "");
  for (my($i)=0; $i<2; $i++) {
    foreach $holiday (@{$holiday[$i]}) {
      if ($d == $holiday) {
        $color = ($i==0)?1:2;
        last;
      }
    }
  }
  if (($this_year == $year) && ($this_month == $month) && ($this_day == $d)) {
    ;
  }
  elsif ($day == $d) {
    ;
  }
  else {
    ;
  }
  if ($caldata_flag) {
    setlocale(LC_ALL, "C");
    $xc_file = strfdate($xc_file_f, $d, $month-1, $year), $xc_file =~ s/xc0/xc/;
    setlocale(LC_ALL, $locale);
  }
  print $OUT "<a href=\"$me?$d+$month+$year\">" if ($caldata_flag && -f $xc_file);
  printf $OUT "<font color=\"$html_colors[$color]\">%-*s</font>", $n_notab, ($d<10)?"0$d":"$d";
  print $OUT "</a>" if ($caldata_flag && -f $xc_file);
}

#
# html_notab_print_the_month(month, year, title of table)
#
sub html_notab_print_the_month {
  my($month, $year, $title) = @_;
  my($leap_year) = is_leap_year($year);
  %hol = xc_holidays($xcalendar_hol, $month, $year);
  %hol2 = xc_holidays($xcalendar_hol2, $month, $year);
  @holiday = holidays($holidays_txt, $month, $year);

  $prev_month = ($month+10)%12 + 1;
  $prev_year = $year - ($prev_month==12?1:0);
  $next_month = $month%12 + 1;
  $next_year = $year + ($next_month==1?1:0);
  print $OUT "<a href=\"$me?$prev_month+$prev_year\"".
      (is_chtml()?" accesskey=\"*\"":is_mml()?" directkey=\"*\"":"").
          ">$prev_month_button</a>" if ($month_navigate_flag);
  print $OUT "$title";
  print $OUT "<a href=\"$me?$next_month+$next_year\"".
      (is_chtml()?" accesskey=\"#\"":is_mml()?" directkey=\"#\"":"").
           ">$next_month_button</a>" if ($month_navigate_flag);
  print $OUT "<br>\n";

  for (my($w)=0; $w<@week_name; $w++) {
    print $OUT "<tt>";
    printf $OUT ($w==0)?"<font color=\"$html_colors[1]\">%-*s</font>":"%-*s", $n_notab, $week_name[$w];
    print $OUT "</tt>";
  }
  print $OUT "<br>\n";
  my($md) = $month_days[$leap_year][$month-1];
  my($wd) = strfdate("%w", 1, $month-1, $year);
  for (my($n, $d)=(0, 0); $n<($wd + $md)/7; $n++) {
    for (my($w)=0; $w<7; $w++) {
      print $OUT "<tt>";
      if ($w < $wd && $d == 0) {
        printf $OUT "%-*s", $n_notab, "--";
      }
      elsif (++$d <= $md) {
        html_notab_print_the_day($d, $w);
      }
      else {
        printf $OUT "%-*s", $n_notab, "--";
      }
      print $OUT "</tt>";
    }
    print $OUT "<br>\n";
  }

  if ($day) {
    if ($caldata_flag) {
      setlocale(LC_ALL, "C");
      $xc_file = strfdate($xc_file_f, $day, $month-1, $year), $xc_file =~ s/xc0/xc/;
      setlocale(LC_ALL, $locale);
    }
    printf $OUT "<br>%s\n", strfdate($date_f, $day, $month-1, $year);
    printf $OUT "%s\n", $hol{$day} if ($holdata_flag && $hol{$day});
    printf $OUT "%s\n", $hol2{$day} if ($holdata_flag && $hol2{$day});
    print $OUT "<br><pre>\n".`/bin/cat $xc_file`."\n</pre>\n" if ($caldata_flag && -f $xc_file);
  }
}

#
# hdml_print_header(title of page)
#
sub hdml_print_header {
  my($title) = @_;
  print $OUT "Content-type: $content_type\n\n" if ($ENV{"HTTP_HOST"});
#  print $OUT "<!doctype hdml public \"-//w3c//dtd hdml 3.0 draft//en\">\n";
  print $OUT <<EOF;
<hdml version="3.0" markable=True>
EOF
}

#
# hdml_print_footer()
#
sub hdml_print_footer {
  print $OUT "</hdml>\n";
}

#
# hdml_print_header_months(title of page)
#
sub hdml_print_header_months {
  my($title) = @_;
  printf $OUT "<choice name=\"$title\" title=\"$title\" method=Alpha key=\"month\"%s>\n", ($this_year == $year)?" default=\"$this_month\"":"";
  print $OUT "$title\n";
  for (my($month)=1; $month<=12; $month++) {
    my($subtitle) = strfdate($subtitle_f, 1, $month-1, $year);
    if (!$divide_deck) {
      printf $OUT "<ce value=\"%s\" task=GOSUB dest=\"#%s\" label=\"OK\">%s\n", "$month", $subtitle, $subtitle;
    }
    else {
      printf $OUT "<ce value=\"%s\" task=GOSUB dest=\"$me?$month+$year\" label=\"OK\">%s\n", "$month", $subtitle;
    }
  }
  $last_year = $year - 1, $next_year = $year + 1;
  printf $OUT "<ce value=\"%s\" task=GOSUB dest=\"$me?$last_year\" label=\"OK\">%s\n", "*", "Last Year";
  printf $OUT "<ce value=\"%s\" task=GOSUB dest=\"$me?$next_year\" label=\"OK\">%s\n", "#", "Next Year";
  print $OUT "</choice>\n";
}

#
# hdml_print_footer_months()
#
sub hdml_print_footer_months {
}

#
# hdml_print_the_day(day of month, week number)
#
sub hdml_print_the_day {
  my($d, $w) = @_;
  my($color, $holiday, $xc_file);
  $color = 0;
  $color = 1 if ($w == 0 || $hol{$d} ne "");
  $color = 2 if ($hol2{$d} ne "");
  for (my($i)=0; $i<2; $i++) {
    foreach $holiday (@{$holiday[$i]}) {
      if ($d == $holiday) {
        $color = ($i==0)?1:2;
        last;
      }
    }
  }
  if (($this_year == $year) && ($this_month == $month) && ($this_day == $d)) {
    ;
  }
  elsif ($day == $d) {
    ;
  }
  else {
    ;
  }
  if ($caldata_flag) {
    setlocale(LC_ALL, "C");
    $xc_file = strfdate($xc_file_f, $d, $month-1, $year), $xc_file =~ s/xc0/xc/;
    setlocale(LC_ALL, $locale);
  }
  print $OUT "<a task=GOSUB dest=\"$me?$d+$month+$year\">" if ($caldata_flag && -f $xc_file);
  printf $OUT ($w==0)?"<i>%-*s</i>":"%-*s", $n_notab, ($d<10)?"0$d":"$d";
#  printf $OUT ($w==0)?"<font color=\"$html_colors[$color]\"><i>%-*s</i></font>":"<font color=\"$html_colors[$color]\">%-*s</font>", $n_notab, ($d<10)?"0$d":"$d";
  print $OUT "</a>" if ($caldata_flag && -f $xc_file);
}

#
# hdml_print_the_month(month, year, title of table)
#
sub hdml_print_the_month {
  my($month, $year, $title) = @_;
  my($leap_year) = is_leap_year($year);
  %hol = xc_holidays($xcalendar_hol, $month, $year);
  %hol2 = xc_holidays($xcalendar_hol2, $month, $year);
  @holiday = holidays($holidays_txt, $month, $year);

  print $OUT "<display name=\"$title\" title=\"$title\">\n";
  print $OUT "<center><line>";
  print $OUT "<i>" if (($this_year == $year) && ($this_month == $month));
  $prev_month = ($month+10)%12 + 1;
  $prev_year = $year - ($prev_month==12?1:0);
  $next_month = $month%12 + 1;
  $next_year = $year + ($next_month==1?1:0);
  print $OUT "<a task=GOSUB dest=\"$me?$prev_month+$prev_year\"".
      (is_hdml()?" accesskey=\"*\"":"").
          ">$prev_month_button</a>" if ($month_navigate_flag);
  print $OUT "$title";
  print $OUT "<a task=GOSUB dest=\"$me?$next_month+$next_year\"".
      (is_hdml()?" accesskey=\"#\"":"").
          ">$next_month_button</a>" if ($month_navigate_flag);
  print $OUT "</i>" if (($this_year == $year) && ($this_month == $month));
  print $OUT "<br>\n";

  print $OUT "<line>";
  for (my($w)=0; $w<@week_name; $w++) {
    printf $OUT ($w==0)?"<i>%-*s</i>":"%-*s", $n_notab, $week_name[$w];
#    printf $OUT ($w==0)?"<font color=\"$html_colors[1]\"><i>%-*s</i></font>":"%-*s", $n_notab, $week_name[$w];
  }
  print $OUT "<br>\n";
  my($md) = $month_days[$leap_year][$month-1];
  my($wd) = strfdate("%w", 1, $month-1, $year);
  for (my($n, $d)=(0, 0); $n<($wd + $md)/7; $n++) {
    print $OUT "<line>";
    for (my($w)=0; $w<7; $w++) {
      if ($w < $wd && $d == 0) {
        printf $OUT "%-*s", $n_notab, "--";
      }
      elsif (++$d <= $md) {
        hdml_print_the_day($d, $w);
      }
      else {
        printf $OUT "%-*s", $n_notab, "--";
      }
    }
    print $OUT "<br>\n";
  }

  print $OUT "<action type=SOFT1 task=GOSUB dest=\"$me?$year\">\n" if (!$months_flag);
  if ($day) {
    if ($caldata_flag) {
      setlocale(LC_ALL, "C");
      $xc_file = strfdate($xc_file_f, $day, $month-1, $year), $xc_file =~ s/xc0/xc/;
      setlocale(LC_ALL, $locale);
    }
    my($date) = strfdate($date_f, $day, $month-1, $year);
    if (($holdata_flag && $hol{$day}) || ($holdata_flag && $hol2{$day}) || ($caldata_flag && -f $xc_file)) {
      print $OUT "<action type=ACCEPT task=GOSUB dest=\"#$date\" label=\"Check\">\n";
      print $OUT "</display>\n";
      print $OUT "<display name=\"$date\" title=\"$date\">\n";
      printf $OUT "%s\n", $hol{$day} if ($holdata_flag && $hol{$day});
      printf $OUT "%s\n", $hol2{$day} if ($holdata_flag && $hol2{$day});
      print $OUT "<br><pre>\n".`/bin/cat $xc_file`."\n</pre>\n" if ($caldata_flag && -f $xc_file);
      print $OUT "<action type=ACCEPT task=PREV label=\"Back\">\n";
    }
    else {
      print $OUT "<action type=ACCEPT task=PREV>\n";
    }
  }
  else {
    print $OUT "<action type=ACCEPT task=PREV>\n";
  }
  print $OUT "</display>\n";
}

#
# ps_print_header(title of document)
#
sub ps_print_header {
  my($title) = @_;
  setlocale(LC_ALL, "C");
  my($time_stamp) = strftime('%c', localtime());
  setlocale(LC_ALL, $ENV{"LANG"});
  my(@_bb) = ps_boundingbox(@bb);
  print $OUT <<EOF;
%!PS-Adobe-3.0
%%Creator: $0, $mailto
%%Version: $version
%%Copyright: $copyright
%%Title: $ps_title
%%CreationDate: $time_stamp
%%BoundingBox: $_bb[0] $_bb[1] $_bb[2] $_bb[3]
%%DocumentData: Clean8Bit
%%DocumentMedia: $medium $media{$medium}[0] $media{$medium}[1] 0 white ()
%%Orientation: $orientations[$orientation]
%%Pages: (atend)
%%PageOrder: Ascend
%%EndComments
%%BeginProlog
/xcalendar_dict 10 dict def
xcalendar_dict begin
/ctext { dup stringwidth pop 2 div neg 0 rmoveto } def
/rtext { dup stringwidth pop neg 0 rmoveto } def
%
% fontname asciifont eucfont euc-font font
%
% References:
% [1] katayama at rd.nacsis.ac.jp, ``fixeucfont,'' sh script, 6/11 1998.
%
/euc-font {
  userdict /euc-font_dict known not {
    userdict begin
      /euc-font_dict 2 dict begin
        /UpperByteEncoding [
          16#00 1 16#20 { pop 0 } for
          16#21 1 16#28 { 16#20 sub } for
          16#29 1 16#2F { pop 0 } for
          16#30 1 16#74 { 16#27 sub } for
          16#75 1 16#FF { pop 0 } for
        ] def
        /LowerByteEncoding [
          16#00 1 16#A0 { pop /.notdef } for
          16#A1 1 16#FE {
            16#80 sub 16 2 string cvrs
            (cXX) dup 1 4 -1 roll
            putinterval cvn
          } for /.notdef
        ] def
        currentdict
      end def
    end
  } if
  12 dict begin
    dup type dup /nametype eq exch /stringtype eq or { findfont } if
    dup /WMode known {
      dup /WMode get /WMode exch def
      WMode 1 eq {
        [ 0 1 -1 0 0 .3 ] makefont
      } if
    } if
    %
    % 7+8 bit EUC font
    %
    12 dict begin
      /7+8-font exch def
      /FontInfo (7+8 bit EUC font) readonly def
      /PaintType 0 def
      /FontType 0 def
      /FontMatrix matrix def
      % /FontName
      /Encoding euc-font_dict /UpperByteEncoding get def
      /FMapType 2 def		% 8/8 Mapping
      7+8-font /WMode known {
        7+8-font /WMode get /WMode exch def
      } {
        /WMode 0 def
      } ifelse
      /FDepVector [
        7+8-font /FDepVector get 0 get
        [ 16#21 1 16#28 {} for 16#30 1 16#74 {} for ] {
          13 dict begin
            /7+8-font 7+8-font def
            /UpperByte exch 16#80 add def
            % /FontName
            /FontInfo (EUC lower byte font) readonly def
            /PaintType 0 def
            /FontType 3 def
            /FontMatrix matrix def
            /FontBBox { 0 0 0 0 } def
            /Encoding euc-font_dict /LowerByteEncoding get def
            % /UniqueID
            % /WMode
            /BuildChar {
              gsave
              exch dup /7+8-font get setfont
              /UpperByte get
              2 string
              dup 0 4 -1 roll put
              dup 1 4 -1 roll put
              dup stringwidth setcharwidth
              0 0 moveto show
              grestore
            } bind def
            currentdict
          end
          /lowerbytefont exch definefont
        } forall
      ] def
      currentdict
    end
    /doublebytefont exch definefont
    exch
    dup type dup /nametype eq exch /stringtype eq or { findfont } if
    exch
    /FDepVector [ 4 2 roll ] def
    /FontType 0 def
    /FMapType 4 def
    /FontMatrix matrix def
    /Encoding [ 0 1 ] def
    /FontBBox { 0 0 0 0 } def
    dup /FontName exch def
    currentdict
  end
  definefont
} def
end
EOF
  if ($charset =~ /^euc-/) {
    print $OUT <<EOF;
xcalendar_dict begin
/$ps_fn_title /$ps_fs_title[0] /$ps_fs_title[1] euc-font pop
/$ps_fn_week /$ps_fs_week[0] /$ps_fs_week[1] euc-font pop
/$ps_fn_day /$ps_fs_day[0] /$ps_fs_day[1] euc-font pop
/$ps_fn_body /$ps_fs_body[0] /$ps_fs_body[1] euc-font pop
/$ps_fn_foot /$ps_fs_foot[0] /$ps_fs_foot[1] euc-font pop
end
%%EndProlog
EOF
  }
  else {
    print $OUT <<EOF;
EOF
  }
}
#
# ps_print_the_day(day of month, week number, the nth week)
#
sub ps_print_the_day {
  my($d, $w, $n) = @_;
  my($color, $holiday, $xc_file);
  $color = 0;
  $color = 1 if ($w == 0 || $hol{$d} ne "");
  $color = 2 if ($hol2{$d} ne "");
  for (my($i)=0; $i<2; $i++) {
    foreach $holiday (@{$holiday[$i]}) {
      if ($d == $holiday) {
        $color = ($i==0)?1:2;
        last;
      }
    }
  }
  if ($ps_color_mode) {
    if ($color == 0) {
      print $OUT "$ps_colors[$color] setgray\n";
    }
    else {
      print $OUT "$ps_colors[$color] setrgbcolor\n";
    }
  }
  my($j) = $n + 1;
  if (!$fold) {
    @cp0 = ($cbb[0] + $w*$day_box_size[0],
            $cbb[3] + (6+1 - $j)*$day_box_size[1]);
  }
  elsif ($fold eq 'saddle') {
    if ($w<4) {
      @cp0 = ($lbb[0] + $w*$day_box_size[0],
              $lbb[3] + (6+1 - $j)*$day_box_size[1]);
    }
    else {
      @cp0 = ($rbb[0] + ($w-4)*$day_box_size[0],
              $rbb[3] + (6+1 - $j)*$day_box_size[1]);
    }
  }
  @cp1 = ($cp0[0] + $ps_day_ratio_box[0]*$day_box_size[0],
          $cp0[1] - $ps_day_ratio_box[1]*$day_box_size[1]);
  if ($color == 1) {
    @cp2 = @cp1;
    $cp2[1] -= $ps_fn_day_size/5;
    $r = 1.4*$ps_fn_day_size/2;
    print $OUT "gsave\n1 setlinewidth\nnewpath\n".
        "@cp2 $r 0 360 arc stroke\ngrestore\n";
  }
  if ($color == 2) {
    @cp2 = @cp1;
    $cp2[1] -= $ps_fn_day_size/5;
    $r = 1.4*$ps_fn_day_size/2;
    print $OUT "gsave\n0.5 setlinewidth\nnewpath\n".
        "@cp2 $r 0 360 arc stroke\ngrestore\n";
  }
  @cp2 = @cp1;
  $cp2[1] -= $ps_fn_day_size/2;
  print $OUT "@cp2 moveto ($d) ctext show\n";
  if ($holdata_flag && ($data="$hol{$d}\n$hol2{$d}") !~ /^\n$/) {
    print $OUT "gsave\n";
    print $OUT "/$ps_fn_body findfont $ps_fn_body_size scalefont setfont\n";
    @cp1 = ($cp0[0] + $ps_day_ratio_box[0]*$day_box_size[0]*2,
            $cp0[1] - $ps_day_ratio_box[1]*$day_box_size[1]);
    @cp2 = @cp1;
    $cp2[1] -= $ps_fn_body_size + $ps_vskp_ratio_H*$day_box_size[1];
    @data = split(/\n+/, $data);
    for (my($i)=0; $i<@data; $i++) {
      $text = ps_text($data[$i]);
      $cp2[1] -= $ps_fn_body_size + $ps_vskp_ratio_H*$day_box_size[1];
      print $OUT "@cp2 moveto ($text) show\n";
    }
    print $OUT "grestore\n";
  }
  if ($caldata_flag) {
    setlocale(LC_ALL, "C");
    $xc_file = strfdate($xc_file_f, $d, $month-1, $year), $xc_file =~ s/xc0/xc/;
    setlocale(LC_ALL, $ENV{"LANG"});
  }
  if ($caldata_flag && -f $xc_file && ($body=`/bin/cat $xc_file`) !~ /^$/) {
    print $OUT "$ps_colors[0] setgray\n";
    print $OUT "gsave\n";
    print $OUT "/$ps_fn_body findfont $ps_fn_body_size scalefont setfont\n";
    @cp1 = ($cp0[0] + $ps_hskp_ratio_W*$day_box_size[0],
            $cp0[1] - $ps_day_ratio_box[1]*$day_box_size[1]*2);
    @cp2 = @cp1;
    $cp2[1] -= $ps_fn_body_size + $ps_vskp_ratio_H*$day_box_size[1];
    @body = split(/\n+/, $body);
    for (my($i)=0; $i<@body; $i++) {
      $text = ps_text($body[$i]);
      $cp2[1] -= $ps_fn_body_size + $ps_vskp_ratio_H*$day_box_size[1];
      print $OUT "@cp2 moveto ($text) show\n";
    }
    print $OUT "grestore\n";
  }
}
#
# ps_print_the_month(month, year, title of table)
#
sub ps_print_the_month {
  my($month, $year, $title) = @_;
  my($leap_year) = is_leap_year($year);
  %hol = xc_holidays($xcalendar_hol, $month, $year);
  %hol2 = xc_holidays($xcalendar_hol2, $month, $year);
  @holiday = holidays($holidays_txt, $month, $year);
  $ps_page++;
  my(@_pbb) = ps_boundingbox(@pbb);
  print $OUT <<EOF;
%%Page: $ps_page $ps_page
%%PageBoundingBox: $_pbb[0] $_pbb[1] $_pbb[2] $_pbb[3]
xcalendar_dict begin
/xcalendar_savedpage save def
$ps_recoord_op
$ps_linewidth setlinewidth
EOF
  print $OUT "$ps_colors[0] setgray\n" if ($ps_color_mode);
  print $OUT "/$ps_fn_week findfont $ps_fn_week_size scalefont setfont\n";
  for (my($w)=0; $w<@week_name; $w++) {
    print $OUT "$ps_colors[1] setrgbcolor\n" if ($ps_color_mode && $w == 0);
    if ($w == 0) {
      if (!$fold) {
        @cp0 = ($cbb[0] + $w*$day_box_size[0], $PBB[1] -
                ($ps_title_ratio_H+$ps_week_ratio_H/2)*($BB[1]-$BB[3]+0));
      }
      elsif ($fold eq 'saddle') {
        if ($w<4) {
          @cp0 = ($lbb[0] + $w*$day_box_size[0], $PBB[1] -
                  ($ps_title_ratio_H+$ps_week_ratio_H/2)*($BB[1]-$BB[3]+0));
        }
        else {
          @cp0 = ($rbb[0] + ($w-4)*$day_box_size[0], $PBB[1] -
                  ($ps_title_ratio_H+$ps_week_ratio_H/2)*($BB[1]-$BB[3]+0));
        }
      }
      @cp1 = ($cp0[0] + $day_box_size[0]/2,
              $cp0[1] - ($ps_vskp_ratio_H/2)*($BB[1]-$BB[3]+0));
      $r = 1.4*$ps_fn_week_size/2;
      print $OUT "gsave\n1 setlinewidth\nnewpath\n".
          "@cp1 $r 0 360 arc stroke\ngrestore\n";
    }
    if (!$fold) {
      @cp0 = ($cbb[0] + $w*$day_box_size[0], $PBB[1] -
              ($ps_title_ratio_H+$ps_week_ratio_H-$ps_vskp_ratio_H/2)
              *($BB[1]-$BB[3]+0));
    }
    elsif ($fold eq 'saddle') {
      if ($w<4) {
        @cp0 = ($lbb[0] + $w*$day_box_size[0], $PBB[1] -
                ($ps_title_ratio_H+$ps_week_ratio_H-$ps_vskp_ratio_H/2)
                *($BB[1]-$BB[3]+0));
      }
      else {
        @cp0 = ($rbb[0] + ($w-4)*$day_box_size[0], $PBB[1] -
                ($ps_title_ratio_H+$ps_week_ratio_H-$ps_vskp_ratio_H/2)
                *($BB[1]-$BB[3]+0));
      }
    }
    @cp1 = ($cp0[0] + $day_box_size[0]/2, $cp0[1]);
    print $OUT "@cp1 moveto ($week_name[$w]) ctext show\n";
    print $OUT "0 setgray\n" if ($ps_color_mode && $w == 0);
  }
  print $OUT "/$ps_fn_day findfont $ps_fn_day_size scalefont setfont\n";
  my($md) = $month_days[$leap_year][$month-1];
  my($wd) = strfdate("%w", 1, $month-1, $year);
  for (my($n, $d)=(0, 0); $n<($wd + $md)/7; $n++) {
    for (my($w)=0; $w<7; $w++) {
      if ($w < $wd && $d == 0) {
        print $OUT "";
      }
      elsif (++$d <= $md) {
        ps_print_the_day($d, $w, $n);
      }
      else {
        print $OUT "";
      }
    }
  }
  print $OUT "0 setgray\n" if ($ps_color_mode);
  if (!$fold) {
    for (my($w)=1; $w<@week_name; $w++) {
      @cp0 = ($cbb[0] + $w*$day_box_size[0], $cbb[1]);
      @cp1 = ($cbb[0] + $w*$day_box_size[0], $cbb[3]);
      print $OUT "@cp0 moveto\n@cp1 lineto\nstroke\n";
    }
    for (my($j)=1; $j<=6; $j++) {
      @cp0 = ($cbb[0], $cbb[3] + (6+1 - $j)*$day_box_size[1]);
      @cp1 = ($cbb[2], $cbb[3] + (6+1 - $j)*$day_box_size[1]);
      print $OUT "@cp0 moveto\n@cp1 lineto\nstroke\n";
    }
  }
  elsif ($fold eq 'saddle') {
    for (my($w)=1; $w<=@week_name; $w++) {
      if ($w < 4) {
        @cp0 = ($lbb[0] + $w*$day_box_size[0], $lbb[1]);
        @cp1 = ($lbb[0] + $w*$day_box_size[0], $lbb[3]);
      }
      elsif ($w == 4) {
        @cp0 = ($lbb[0] + $w*$day_box_size[0], $lbb[1]);
        @cp1 = ($lbb[0] + $w*$day_box_size[0], $lbb[3]);
        print $OUT "@cp0 moveto\n@cp1 lineto\nstroke\n";
        @cp0 = ($rbb[0] + ($w-4)*$day_box_size[0], $rbb[1]);
        @cp1 = ($rbb[0] + ($w-4)*$day_box_size[0], $rbb[3]);
      }
      else {
        @cp0 = ($rbb[0] + ($w-5)*$day_box_size[0], $rbb[1]);
        @cp1 = ($rbb[0] + ($w-5)*$day_box_size[0], $rbb[3]);
      }
      print $OUT "@cp0 moveto\n@cp1 lineto\nstroke\n";
    }
    for (my($j)=1; $j<=6; $j++) {
      @cp0 = ($lbb[0], $lbb[3] + (6+1 - $j)*$day_box_size[1]);
      @cp1 = ($lbb[2], $lbb[3] + (6+1 - $j)*$day_box_size[1]);
      print $OUT "@cp0 moveto\n@cp1 lineto\nstroke\n";
      @cp0 = ($rbb[0], $rbb[3] + (6+1 - $j)*$day_box_size[1]);
      @cp1 = ($rbb[2], $rbb[3] + (6+1 - $j)*$day_box_size[1]);
      print $OUT "@cp0 moveto\n@cp1 lineto\nstroke\n";
    }
  }
  if (!$fold) {
    print $OUT <<EOF;
/$ps_fn_title findfont $ps_fn_title_size scalefont setfont
$ps_title_pos[0] $ps_title_pos[1] moveto ($title) show

newpath
$cbb[0] $cbb[1] moveto
$cbb[2] $cbb[1] lineto
$cbb[2] $cbb[3] lineto
$cbb[0] $cbb[3] lineto
$cbb[0] $cbb[1] lineto
closepath
stroke

newpath
$PBB[0] $PBB[1] moveto
$PBB[2] $PBB[1] lineto
$PBB[2] $PBB[3] lineto
$PBB[0] $PBB[3] lineto
$PBB[0] $PBB[1] lineto
closepath
stroke
EOF
  }
  elsif ($fold eq 'saddle') {
    print $OUT <<EOF;
/$ps_fn_title findfont $ps_fn_title_size scalefont setfont
$ps_title_pos[0] $ps_title_pos[1] moveto ($title) show

newpath
$lbb[0] $lbb[1] moveto
$lbb[2] $lbb[1] lineto
$lbb[2] $lbb[3] lineto
$lbb[0] $lbb[3] lineto
$lbb[0] $lbb[1] lineto
closepath
stroke

newpath
$rbb[0] $rbb[1] moveto
$rbb[2] $rbb[1] lineto
$rbb[2] $rbb[3] lineto
$rbb[0] $rbb[3] lineto
$rbb[0] $rbb[1] lineto
closepath
stroke

newpath
$PBB[0] $PBB[1] moveto
$PBB[2] $PBB[1] lineto
$PBB[2] $PBB[3] lineto
$PBB[0] $PBB[3] lineto
$PBB[0] $PBB[1] lineto
closepath
stroke
EOF
    print $OUT <<EOF if ($foldline); # -foldline

gsave
[ 2 2 ] 0 setdash
$ps_linewidth setlinewidth
newpath
$DBB[2] $DBB[0] sub 2 div $DBB[1] moveto
$DBB[2] $DBB[0] sub 2 div $DBB[3] lineto
closepath
stroke
grestore
EOF
  }
  print $OUT <<EOF if ($cutline); # -cutline

gsave
0 setlinewidth
newpath
$DBB[0] $DBB[1] moveto
$DBB[2] $DBB[1] lineto
$DBB[2] $DBB[3] lineto
$DBB[0] $DBB[3] lineto
$DBB[0] $DBB[1] lineto
closepath
stroke
grestore
EOF
  $foot = "$year-$month";
  print $OUT <<EOF;

/$ps_fn_foot findfont $ps_fn_foot_size scalefont setfont
$ps_foot_pos[0] $ps_foot_pos[1] moveto ($foot) rtext show
xcalendar_savedpage restore
end
showpage
EOF
}

#
# ps_print_footer()
#
sub ps_print_footer {
  print $OUT <<EOF;
%%Trailer
%%EOF
EOF
}

#
# ps_text(string)
#
sub ps_text {
  my($buf) = @_;
  $buf =~ s/\\/\\\\/g, $buf =~ s/\)/\\)/g, $buf =~ s/\(/\\(/g;
  $buf;
}

#
# ps_boundingbox(left, top, right, bottom)
#
sub ps_boundingbox {
  my($l, $t, $r, $b) = @_;
  $l += $ps_device_origin{$ps_printer_name}[0];
  $t += $ps_device_origin{$ps_printer_name}[1];
  $r += $ps_device_origin{$ps_printer_name}[0];
  $b += $ps_device_origin{$ps_printer_name}[1];
  (($l<$r)?$l:$r-1, ($t<$b)?$t:$b-1, ($l>$r)?$l:$r+1, ($t>$b)?$t:$b+1);
}
